import requests as request
from config import Config
from constantes import URL


class Api(Config):

    def __init__(self, config):
        super().__init__(config)

    def __get_url(self, find_params):
        lang = self.get_lang()
        units = self.get_units()
        api_key = self.get_api_key()
        params = f'{find_params}{lang}{units}{api_key}'
        url = f'{URL}{self.get_type()}{params}'
        return url

    def get_data(self, find_params):
        "Obtener los datos con la url generada"
        return request.get(self.__get_url(find_params)).json()



# url API
URL = 'https://api.openweathermap.org/data/2.5/'

# Opciones de reporte (Actual o previsión de los próximos 5 días)
ACTUAL = 'weather?'
PREVISION = 'forecast?'

# Códigos de lenguajes que se pueden pasar a la API.
CODIGO_LENGUAJE = {
    'es': 'Spanish',
    'en': 'English'
}



from constantes import ACTUAL, PREVISION


class Config:

    def __init__(self, config):
        self.set_api_key(config['key'])
        self.set_units(config['units'])
        self.set_lang(config['lang'])
        self.set_type(config['type'])

    def set_api_key(self, key):
        if key == '' or key == None:
            raise ValueError('Debes especificar un api key')
        self.__api_key = f'&appid={key}'

    def get_api_key(self):
        return self.__api_key

    # Si no se recibe celcius (El que deseamos en nuestro caso)
    # dejamos vacio para que tome el por defecto, que es kelvin.
    def set_units(self, units):
        self.__units = '&units=metric' if units == 'metric' else ''

    def get_units(self):
        return self.__units

    # por defecto se deja inglés, si no, el que pasen.
    def set_lang(self, lang):
        self.__lang = '&lang={}'.format('en' if lang == '' or lang == None else lang)

    def get_lang(self):
        return self.__lang

    def set_type(self, type):
        if type != ACTUAL and type != PREVISION:
            raise ValueError('Debe especificar un tipo correcto. Actual o Prevision')
        self.__type = type

    def get_type(self):
        return self.__type



from api import Api


class Ciudad(Api):

    def __init__(self, config=None, ciudad=None):
        if ciudad == '' or ciudad == None:
            raise ValueError('Nombre de la ciudad requerido')

        if config == None:
            raise ValueError('Configuración requerida')

        self.set_nombre(ciudad)
        super().__init__(config)

    def set_nombre(self, ciudad):
        self.__nombre = ciudad

    def get_nombre(self):
        return self.__nombre

    def get_data(self):
        return super().get_data(f'q={self.__nombre}')



from api import Api


class Coordenadas(Api):

    def __init__(self, config=None, latitud=None, longitud=None):
        if latitud == '' or latitud == None or longitud == '' or longitud == None:
            raise ValueError('Se requiere latitud y longitud')

        if config == None:
            raise ValueError('Configuración requerida')

        self.set_latitud(latitud)
        self.set_longitud(longitud)
        super().__init__(config)

    def set_latitud(self, latitud):
        self.__latitud = latitud

    def get_latitud(self):
        return self.__latitud

    def set_longitud(self, longitud):
        self.__longitud = longitud

    def get_longitud(self):
        return self.__longitud

    def get_data(self):
        return super().get_data('lat={}&lon={}'.format(self.__latitud, self.__longitud))
